# Generated by Django 2.2.9 on 2020-01-3 13:58

from django.db import migrations
from django_netjsongraph.migrations import migrate_addresses

migrate_addresses.__defaults__ = ('topology',)


class Migration(migrations.Migration):

    dependencies = [
        ('topology', '0007_create_new_address_field'),
    ]

    operations = [
        migrations.RunPython(migrate_addresses)
    ]



# Generated by Django 2.0.2 on 2018-02-19 14:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('topology', '0002_snapshot'),
    ]

    operations = [
        migrations.AddField(
            model_name='link',
            name='status_changed',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='topology',
            name='parser',
            field=models.CharField(choices=[('netdiff.OlsrParser', 'OLSRd (txtinfo/jsoninfo)'), ('netdiff.BatmanParser', 'batman-advanced (jsondoc/txtinfo)'), ('netdiff.BmxParser', 'BMX6 (q6m)'), ('netdiff.NetJsonParser', 'NetJSON NetworkGraph'), ('netdiff.CnmlParser', 'CNML 1.0'), ('netdiff.OpenvpnParser', 'OpenVPN')], help_text='Select topology format', max_length=128, verbose_name='format'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-13 13:31
 
import collections
import re
import uuid

import django.core.validators
import django.db.models.deletion
import django.utils.timezone
import django_netjsongraph.utils
import jsonfield.fields
import model_utils.fields
from django.db import migrations, models

import openwisp_utils


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('openwisp_users', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Link',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('cost', models.FloatField()),
                ('cost_text', models.CharField(blank=True, max_length=24)),
                ('status', model_utils.fields.StatusField(choices=[('up', 'up'), ('down', 'down')], default='up', max_length=100, no_check_for_status=True)),
                ('properties', jsonfield.fields.JSONField(blank=True, default=dict, dump_kwargs={'indent': 4}, load_kwargs={'object_pairs_hook': collections.OrderedDict})),
                ('organization', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openwisp_users.Organization', verbose_name='organization')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Node',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(blank=True, max_length=64)),
                ('addresses', models.CharField(db_index=True, max_length=510)),
                ('properties', jsonfield.fields.JSONField(blank=True, default=dict, dump_kwargs={'indent': 4}, load_kwargs={'object_pairs_hook': collections.OrderedDict})),
                ('organization', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openwisp_users.Organization', verbose_name='organization')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Topology',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(max_length=64, verbose_name='label')),
                ('parser', models.CharField(choices=[('netdiff.OlsrParser', 'OLSRd (txtinfo/jsoninfo)'), ('netdiff.BatmanParser', 'batman-advanced (jsondoc/txtinfo)'), ('netdiff.BmxParser', 'BMX6 (q6m)'), ('netdiff.NetJsonParser', 'NetJSON NetworkGraph'), ('netdiff.CnmlParser', 'CNML 1.0')], help_text='Select topology format', max_length=128, verbose_name='format')),
                ('strategy', models.CharField(choices=[('fetch', 'FETCH'), ('receive', 'RECEIVE')], db_index=True, default='fetch', max_length=16, verbose_name='strategy')),
                ('url', models.URLField(blank=True, help_text='Topology data will be fetched from this URL (FETCH strategy)', verbose_name='url')),
                ('key', openwisp_utils.base.KeyField(blank=True, default=openwisp_utils.utils.get_random_key, help_text='key needed to update topology from nodes ', max_length=64, validators=[django.core.validators.RegexValidator(re.compile('^[^\\s/\\.]+$'), code='invalid', message='This value must not contain spaces, dots or slashes.')], verbose_name='key')),
                ('expiration_time', models.PositiveIntegerField(default=0, help_text='"Expiration Time" in seconds: setting this to 0 will immediately mark missing links as down; a value higher than 0 will delay marking missing links as down until the "modified" field of a link is older than "Expiration Time"', verbose_name='expiration time')),
                ('published', models.BooleanField(default=True, help_text="Unpublished topologies won't be updated or shown in the visualizer", verbose_name='published')),
                ('protocol', models.CharField(blank=True, max_length=64, verbose_name='protocol')),
                ('version', models.CharField(blank=True, max_length=24, verbose_name='version')),
                ('revision', models.CharField(blank=True, max_length=64, verbose_name='revision')),
                ('metric', models.CharField(blank=True, max_length=24, verbose_name='metric')),
                ('organization', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openwisp_users.Organization', verbose_name='organization')),
            ],
            options={
                'verbose_name_plural': 'topologies',
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='node',
            name='topology',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='topology.Topology'),
        ),
        migrations.AddField(
            model_name='link',
            name='source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='source_link_set', to='topology.Node'),
        ),
        migrations.AddField(
            model_name='link',
            name='target',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='source_target_set', to='topology.Node'),
        ),
        migrations.AddField(
            model_name='link',
            name='topology',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='topology.Topology'),
        ),
    ]



VERSION = (0, 3, 1, 'final')
__version__ = VERSION  # alias


def get_version():
    version = '%s.%s' % (VERSION[0], VERSION[1])
    if VERSION[2]:
        version = '%s.%s' % (version, VERSION[2])
    if VERSION[3:] == ('alpha', 0):
        version = '%s pre-alpha' % version
    else:
        if VERSION[3] != 'final':
            try:
                rev = VERSION[4]
            except IndexError:
                rev = 0
            version = '%s%s%s' % (version, VERSION[3][0:1], rev)
    return version


default_app_config = 'openwisp_network_topology.apps.OpenwispNetworkTopologyConfig'



from django.db import migrations


def reformat_address_forward(apps, schema_editor):
    Node = apps.get_model('topology', 'Node')
    for node in Node.objects.all():
        if not node.addresses.startswith(';'):
            node.addresses = ';{0}'.format(node.addresses)
            node.save()


def reformat_address_backward(apps, schema_editor):
    Fake_node_model = apps.get_model('topology', 'Node')
    for node in Fake_node_model.objects.all():
        if node.addresses.startswith(';'):
            node.addresses = node.addresses[1:]
            node.save()


class Migration(migrations.Migration):

    dependencies = [
        ('topology', '0005_default_operator_permission'),
    ]

    operations = [
        migrations.RunPython(reformat_address_forward, reformat_address_backward)
    ]



from openwisp_users.models import Organization


class CreateOrgMixin(object):
    def _create_org(self, **kwargs):
        options = dict(name='test-organization')
        options.update(kwargs)
        org = Organization(**options)
        org.save()
        return org



from django_netjsongraph.api.generics import (BaseNetworkCollectionView, BaseNetworkGraphHistoryView,
                                              BaseNetworkGraphView, BaseReceiveTopologyView)

from ..models import Snapshot, Topology


class NetworkCollectionView(BaseNetworkCollectionView):
    queryset = Topology.objects.filter(published=True)


class NetworkGraphView(BaseNetworkGraphView):
    queryset = Topology.objects.filter(published=True)


class ReceiveTopologyView(BaseReceiveTopologyView):
    model = Topology


class NetworkGraphHistoryView(BaseNetworkGraphHistoryView):
    topology_model = Topology
    snapshot_model = Snapshot


network_collection = NetworkCollectionView.as_view()
network_graph = NetworkGraphView.as_view()
network_graph_history = NetworkGraphHistoryView.as_view()
receive_topology = ReceiveTopologyView.as_view()



from django.test import TestCase
from django_netjsongraph.tests import CreateGraphObjectsMixin
from django_netjsongraph.tests.base.test_link import TestLinkMixin

from ..models import Link, Node, Topology
from . import CreateOrgMixin


class TestLink(TestLinkMixin, CreateOrgMixin,
               CreateGraphObjectsMixin, TestCase):
    topology_model = Topology
    node_model = Node
    link_model = Link

    def setUp(self):
        org = self._create_org()
        t = self._create_topology(organization=org)
        self._create_node(label="node1", addresses=["192.168.0.1"],
                          topology=t, organization=org)
        self._create_node(label="node2", addresses=["192.168.0.2"],
                          topology=t, organization=org)



from django.contrib import admin
from django_netjsongraph.base.admin import AbstractLinkAdmin, AbstractNodeAdmin, AbstractTopologyAdmin

from openwisp_users.multitenancy import (MultitenantAdminMixin, MultitenantOrgFilter,
                                         MultitenantRelatedOrgFilter)

from .models import Link, Node, Topology


class TopologyAdmin(MultitenantAdminMixin, AbstractTopologyAdmin):
    model = Topology


TopologyAdmin.list_display += ('organization',)
TopologyAdmin.list_filter += (('organization', MultitenantOrgFilter),)
TopologyAdmin.fields.insert(1, 'organization')


class NodeAdmin(MultitenantAdminMixin, AbstractNodeAdmin):
    model = Node
    list_display = ['name', 'organization', 'topology', 'addresses']
    list_filter = [('organization', MultitenantOrgFilter),
                   ('topology', MultitenantRelatedOrgFilter)]
    multitenant_shared_relations = ('topology',)
    fields = ['label', 'organization', 'addresses', 'properties', 'topology',
              'created', 'modified']


class LinkAdmin(MultitenantAdminMixin, AbstractLinkAdmin):
    model = Link
    list_display = ['__str__', 'organization', 'topology', 'status', 'cost',
                    'cost_text']
    list_filter = ['status',
                   ('organization', MultitenantOrgFilter),
                   ('topology', MultitenantRelatedOrgFilter)]
    multitenant_shared_relations = ('topology', 'source', 'target')
    fields = ['organization', 'cost', 'cost_text', 'status', 'properties',
              'topology', 'source', 'target', 'created', 'modified']


admin.site.register(Link, LinkAdmin)
admin.site.register(Node, NodeAdmin)
admin.site.register(Topology, TopologyAdmin)



# Generated by Django 2.0.2 on 2018-02-20 22:54

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('topology', '0003_link_status_and_openvpn_parser'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='target',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='target_link_set', to='topology.Node'),
        ),
    ]



from django.conf.urls import include, url

from .api import urls as api
from .visualizer import urls as visualizer_urls

urlpatterns = [
    url(r'^accounts/', include('openwisp_users.accounts.urls')),
    url(r'^api/', include(api)),
    url(r'^topology/', include(visualizer_urls)),
]



from django.test import TestCase
from django.urls import reverse
from django_netjsongraph.tests import CreateGraphObjectsMixin
from django_netjsongraph.tests.base.test_admin import TestAdminMixin

from openwisp_users.tests.utils import TestMultitenantAdminMixin, TestOrganizationMixin

from ..apps import OpenwispNetworkTopologyConfig as appconfig
from ..models import Link, Node, Topology
from . import CreateOrgMixin


class TestAdmin(CreateGraphObjectsMixin, CreateOrgMixin,
                TestAdminMixin, TestCase):
    topology_model = Topology
    link_model = Link
    node_model = Node
    module = 'openwisp_network_topology'

    @property
    def prefix(self):
        return 'admin:{0}'.format(appconfig.label)

    def setUp(self):
        org = self._create_org()
        t = self._create_topology(organization=org)
        self._create_node(label="node1",
                          addresses=["192.168.0.1"],
                          topology=t,
                          organization=org)
        self._create_node(label="node2",
                          addresses=["192.168.0.2"],
                          topology=t,
                          organization=org)
        super().setUp()


class TestMultitenantAdmin(CreateGraphObjectsMixin, TestMultitenantAdminMixin,
                           TestOrganizationMixin, TestCase):
    topology_model = Topology
    node_model = Node
    link_model = Link
    operator_permission_filters = [
        {'codename__endswith': 'topology'},
        {'codename__endswith': 'node'},
        {'codename__endswith': 'link'},
    ]

    def _create_multitenancy_test_env(self):
        org1 = self._create_org(name='test1org')
        org2 = self._create_org(name='test2org')
        inactive = self._create_org(name='inactive-org', is_active=False)
        operator = self._create_operator(organizations=[org1, inactive])
        t1 = self._create_topology(label='topology1org', organization=org1)
        t2 = self._create_topology(label='topology2org', organization=org2)
        t3 = self._create_topology(label='topology3org', organization=inactive)
        n11 = self._create_node(label='node1org1', topology=t1, organization=org1)
        n12 = self._create_node(label='node2org1', topology=t1, organization=org1)
        n21 = self._create_node(label='node1org2', topology=t2, organization=org2)
        n22 = self._create_node(label='node2org2', topology=t2, organization=org2)
        n31 = self._create_node(label='node1inactive', topology=t3, organization=inactive)
        n32 = self._create_node(label='node2inactive', topology=t3, organization=inactive)
        l1 = self._create_link(topology=t1,
                               organization=org1,
                               source=n11,
                               target=n12)
        l2 = self._create_link(topology=t2,
                               organization=org2,
                               source=n21,
                               target=n22)
        l3 = self._create_link(topology=t3,
                               organization=inactive,
                               source=n31,
                               target=n32)
        data = dict(t1=t1, t2=t2, t3_inactive=t3,
                    n11=n11, n12=n12, l1=l1,
                    n21=n21, n22=n22, l2=l2,
                    n31=n31, n32=n32, l3_inactive=l3,
                    org1=org1, org2=org2,
                    inactive=inactive,
                    operator=operator)
        return data

    def test_topology_queryset(self):
        data = self._create_multitenancy_test_env()
        self._test_multitenant_admin(
            url=reverse('admin:topology_topology_changelist'),
            visible=[data['t1'].label, data['org1'].name],
            hidden=[data['t2'].label, data['org2'].name,
                    data['t3_inactive'].label]
        )

    def test_topology_organization_fk_queryset(self):
        data = self._create_multitenancy_test_env()
        self._test_multitenant_admin(
            url=reverse('admin:topology_topology_add'),
            visible=[data['org1'].name],
            hidden=[data['org2'].name, data['inactive']],
            select_widget=True
        )

    def test_node_queryset(self):
        data = self._create_multitenancy_test_env()
        self._test_multitenant_admin(
            url=reverse('admin:topology_node_changelist'),
            visible=[data['n11'].label, data['n12'].label, data['org1'].name],
            hidden=[data['n21'].label, data['n22'].label, data['org2'].name,
                    data['n31'].label, data['n32'].label, data['inactive']]
        )

    def test_node_organization_fk_queryset(self):
        data = self._create_multitenancy_test_env()
        self._test_multitenant_admin(
            url=reverse('admin:topology_node_add'),
            visible=[data['org1'].name],
            hidden=[data['org2'].name, data['inactive']],
            select_widget=True
        )

    def test_link_queryset(self):
        data = self._create_multitenancy_test_env()
        self._test_multitenant_admin(
            url=reverse('admin:topology_link_changelist'),
            visible=[str(data['l1']), data['org1'].name],
            hidden=[str(data['l2']), data['org2'].name,
                    str(data['l3_inactive'])]
        )

    def test_link_organization_fk_queryset(self):
        data = self._create_multitenancy_test_env()
        self._test_multitenant_admin(
            url=reverse('admin:topology_link_add'),
            visible=[data['org1'].name],
            hidden=[data['org2'].name, data['inactive']],
            select_widget=True
        )

    def test_node_topology_fk_queryset(self):
        data = self._create_multitenancy_test_env()
        self._test_multitenant_admin(
            url=reverse('admin:topology_node_add'),
            visible=[data['t1'].label],
            hidden=[data['t2'].label, data['t3_inactive'].label]
        )

    def test_link_topology_fk_queryset(self):
        data = self._create_multitenancy_test_env()
        self._test_multitenant_admin(
            url=reverse('admin:topology_link_add'),
            visible=[data['t1'].label],
            hidden=[data['t2'].label, data['t3_inactive'].label]
        )

    def test_node_topology_filter(self):
        data = self._create_multitenancy_test_env()
        t_special = self._create_topology(label='special', organization=data['org1'])
        self._test_multitenant_admin(
            url=reverse('admin:topology_node_changelist'),
            visible=[data['t1'].label, t_special.label],
            hidden=[data['t2'].label, data['t3_inactive'].label]
        )

    def test_link_topology_filter(self):
        data = self._create_multitenancy_test_env()
        t_special = self._create_topology(label='special', organization=data['org1'])
        self._test_multitenant_admin(
            url=reverse('admin:topology_link_changelist'),
            visible=[data['t1'].label, t_special.label],
            hidden=[data['t2'].label, data['t3_inactive'].label]
        )



from django.test import TestCase
from django_netjsongraph.tests import CreateGraphObjectsMixin
from django_netjsongraph.tests.utils import TestUtilsMixin

from openwisp_users.models import Organization

from ..models import Link, Node, Snapshot, Topology
from . import CreateOrgMixin


class TestUtils(TestCase, TestUtilsMixin,
                CreateGraphObjectsMixin, CreateOrgMixin):
    topology_model = Topology
    node_model = Node
    link_model = Link
    snapshot_model = Snapshot

    def setUp(self):
        org = self._create_org()
        t = self._create_topology(organization=org)
        self._create_node(label="node1", addresses=["192.168.0.1"],
                          topology=t, organization=org)
        self._create_node(label="node2", addresses=["192.168.0.2"],
                          topology=t, organization=org)

    def _create_link(self, **kwargs):
        org = Organization.objects.first()
        options = dict(cost='1',
                       cost_text='one',
                       properties={},
                       organization=org)
        options.update(kwargs)
        link = self.link_model(**options)
        link.full_clean()
        link.save()
        return link

    def test_save_snapshot_all_method(self, **kwargs):
        org = self._create_org()
        options = dict(organization=org)
        options.update(**kwargs)
        super().test_save_snapshot_all_method(**options)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-15 10:01

import uuid

import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('topology', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Snapshot',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('data', models.TextField()),
                ('date', models.DateField(auto_now=True)),
                ('organization', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openwisp_users.Organization', verbose_name='organization')),
                ('topology', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='topology.Topology')),
            ],
            options={
                'abstract': False,
                'verbose_name_plural': 'snapshots',
            },
        ),
    ]



# Generated by Django 2.2.9 on 2020-01-3 14:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('topology', '0008_migrate_addresses_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='node',
            name='addresses_old',
        ),
    ]



from django.conf import settings
from django_netjsongraph.apps import DjangoNetjsongraphConfig


class OpenwispNetworkTopologyConfig(DjangoNetjsongraphConfig):
    name = 'openwisp_network_topology'
    label = 'topology'

    def ready(self, *args, **kwargs):
        super().ready(*args, **kwargs)
        self.add_default_menu_items()

    def add_default_menu_items(self):
        menu_setting = 'OPENWISP_DEFAULT_ADMIN_MENU_ITEMS'
        items = [
            {'model': 'topology.Topology'}
        ]
        if not hasattr(settings, menu_setting):
            setattr(settings, menu_setting, items)
        else:
            current_menu = getattr(settings, menu_setting)
            current_menu += items



from django.db import models
from django_netjsongraph.base.link import AbstractLink
from django_netjsongraph.base.node import AbstractNode
from django_netjsongraph.base.snapshot import AbstractSnapshot
from django_netjsongraph.base.topology import AbstractTopology

from openwisp_users.mixins import OrgMixin


class Link(OrgMixin, AbstractLink):
    topology = models.ForeignKey('topology.Topology',
                                 on_delete=models.CASCADE)
    source = models.ForeignKey('topology.Node',
                               related_name='source_link_set',
                               on_delete=models.CASCADE)
    target = models.ForeignKey('topology.Node',
                               related_name='target_link_set',
                               on_delete=models.CASCADE)

    class Meta(AbstractLink.Meta):
        abstract = False


class Node(OrgMixin, AbstractNode):
    topology = models.ForeignKey('topology.Topology',
                                 on_delete=models.CASCADE)

    class Meta(AbstractNode.Meta):
        abstract = False


class Snapshot(OrgMixin, AbstractSnapshot):
    topology = models.ForeignKey('topology.Topology',
                                 on_delete=models.CASCADE)

    class Meta(AbstractSnapshot.Meta):
        abstract = False


class Topology(OrgMixin, AbstractTopology):
    def _create_node(self, **kwargs):
        options = dict(organization=self.organization,
                       topology=self)
        options.update(kwargs)
        return self.node_model(**options)

    def _create_link(self, **kwargs):
        options = dict(organization=self.organization,
                       topology=self)
        options.update(kwargs)
        return self.link_model(**options)

    def save_snapshot(self, **kwargs):
        options = dict(organization=self.organization)
        options.update(kwargs)
        super().save_snapshot(**options)

    class Meta(AbstractTopology.Meta):
        abstract = False



from django.contrib.auth.management import create_permissions
from django.contrib.auth.models import Permission
from django.db import migrations


def create_default_permissions(apps, schema_editor):
    for app_config in apps.get_app_configs():
        app_config.models_module = True
        create_permissions(app_config, apps=apps, verbosity=0)
        app_config.models_module = None


def assign_permissions_to_groups(apps, schema_editor):
    create_default_permissions(apps, schema_editor)
    operators_and_admin_can_change = ['link', 'node']
    operators_read_only_admins_manage = ['topology']
    manage_operations = ['add', 'change', 'delete']
    Group = apps.get_model('openwisp_users', 'Group')

    try:
        admin = Group.objects.get(name='Administrator')
        operator = Group.objects.get(name='Operator')
    # consider failures custom cases
    # that do not have to be dealt with
    except Group.DoesNotExist:
        return

    for model_name in operators_and_admin_can_change:
        for operation in manage_operations:
            permission = Permission.objects.get(
                codename='{}_{}'.format(operation, model_name)
            )
            admin.permissions.add(permission.pk)
            operator.permissions.add(permission.pk)
    for model_name in operators_read_only_admins_manage:
        try:
            permission = Permission.objects.get(
                codename="view_{}".format(model_name)
            )
            operator.permissions.add(permission.pk)
        except Permission.DoesNotExist:
            pass        
        for operation in manage_operations:
            admin.permissions.add(
                Permission.objects.get(codename="{}_{}".format(operation, model_name)).pk
            )


class Migration(migrations.Migration):
    dependencies = [
        ('openwisp_users', '0004_default_groups'),
        ('topology', '0004_fixed_target_link_set')
    ]
    operations = [
        migrations.RunPython(
            assign_permissions_to_groups,
            reverse_code=migrations.RunPython.noop
        )
    ]



# Generated by Django 2.2.9 on 2020-01-3 13:52

import jsonfield.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('topology', '0006_reformat_addresses'),
    ]

    operations = [
        migrations.RenameField(
            model_name='node',
            old_name='addresses',
            new_name='addresses_old',
        ),
        migrations.AddField(
            model_name='node',
            name='addresses',
            field=jsonfield.fields.JSONField(default=[]),
        ),
    ]



from django_netjsongraph.visualizer.generics import BaseTopologyDetailView, BaseTopologyListView

from ..models import Topology


class TopologyListView(BaseTopologyListView):
    topology_model = Topology


class TopologyDetailView(BaseTopologyDetailView):
    topology_model = Topology


topology_list = TopologyListView.as_view()
topology_detail = TopologyDetailView.as_view()
